package dev.fitko.fitconnect.api.config;

import dev.fitko.fitconnect.api.exceptions.InitializationException;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Collections;
import java.util.Map;
import java.util.stream.Collectors;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class ApplicationConfig {

    @Builder.Default
    private String httpProxyHost = "";

    @Builder.Default
    private Integer httpProxyPort = 0;

    @Builder.Default
    private Integer requestTimeoutInSeconds = 30;

    @Builder.Default
    private String metadataSchemaPath = "";

    @Builder.Default
    private String privateSigningKeyPath = "";

    private SenderConfig senderConfig;

    private SubscriberConfig subscriberConfig;

    @Builder.Default
    private Map<EnvironmentName, Environment> environments = Collections.emptyMap();

    private EnvironmentName activeEnvironment;

    private ResourcePaths resourcePaths;

    public Environment getEnvironmentByName(final EnvironmentName environmentName) {
        if (environments.containsKey(environmentName)) {
            return environments.get(environmentName);
        } else {
            throw new InitializationException("No environment with name '" + environmentName.getName() + "' found. Available environments are: " + getAvailableEnvironmentNames());
        }
    }

    public Environment getCurrentEnvironment() {
        return getEnvironmentByName(activeEnvironment);
    }

    public String getOAuthTokenEndpoint() {
        return getCurrentEnvironment().getAuthBaseUrl() + resourcePaths.getAuthTokenPath();
    }

    public String getSubmissionsEndpoint() {
        return getSubmissionBaseUrl() + resourcePaths.getSubmissionsPath();
    }

    public String getSubmissionEndpoint() {
        return getSubmissionBaseUrl() + resourcePaths.getSubmissionPath();
    }

    public String getAttachmentEndpoint() {
        return getSubmissionBaseUrl() + resourcePaths.getSubmissionAttachmentPath();
    }

    public String getEventLogEndpoint() {
        return getSubmissionBaseUrl() + resourcePaths.getEventLogPath();
    }

    public String destinationEndpoint() {
        return getSubmissionBaseUrl() + resourcePaths.getDestinationPath();
    }

    public String destinationKeyEndpoint() {
        return getSubmissionBaseUrl() + resourcePaths.getDestinationKeyPath();
    }

    private String getSubmissionBaseUrl() {
        return getCurrentEnvironment().getSubmissionBaseUrl();
    }

    private String getAvailableEnvironmentNames() {
        return environments.keySet()
                .stream()
                .map(EnvironmentName::getName)
                .collect(Collectors.joining(" | "));
    }
}

