package dev.fitko.fitconnect.api.services.events;

import dev.fitko.fitconnect.api.domain.model.event.EventLog;
import dev.fitko.fitconnect.api.domain.model.submission.Submission;

import java.util.UUID;

/**
 *  A service that creates SET events that are added to a {@link Submission}s {@link EventLog}.
 *
 * @see EventLogService
 * @see <a href="https://docs.fitko.de/fit-connect/docs/getting-started/event-log/events">Event Documentation</a>
 */
public interface SecurityEventService {

    /**
     * Create a new accept_submission event for a valid submission.
     *
     * @param submissionId unique submission identifier
     * @param destinationId unique destination identifier
     * @param caseId unique case identifier
     *
     * @return signed and SET encoded event
     */
    String createAcceptSubmissionEvent(UUID submissionId, UUID destinationId, UUID caseId);

    /**
     * Create a new reject_submission event for an invalid submission.
     *
     * @param submissionId unique submission identifier
     * @param destinationId unique destination identifier
     * @param caseId unique case identifier
     *
     * @return signed and SET encoded event
     */
    String createRejectSubmissionEvent(UUID submissionId, UUID destinationId, UUID caseId);

}
