package dev.fitko.fitconnect.client.cmd;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.client.SenderClient;
import dev.fitko.fitconnect.client.SubscriberClient;
import dev.fitko.fitconnect.client.factory.ApplicationConfigLoader;
import dev.fitko.fitconnect.client.factory.ClientFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.stream.Collectors;

public final class CommandLineRunner {

    private static final String LOGO = "/splash_screen_banner.txt";
    public static final String DEFAULT_CONFIG_NAME = "config.yml";

    private static final Logger LOGGER = LoggerFactory.getLogger(CommandLineRunner.class);

    private CommandLineRunner() {
    }

    public static void main(final String[] args) throws IOException {
        printSplashScreen();
        run(args);
    }

    private static void run(final String[] args) throws IOException {
        final ApplicationConfig config = ApplicationConfigLoader.loadConfig(DEFAULT_CONFIG_NAME);
        final var senderClient = getSenderClient(config);
        final var subscriberClient = getSubscriberClient(config);
        new CommandLineClient(senderClient, subscriberClient).run(args);
    }

    private static SubscriberClient.RequestSubmission getSubscriberClient(final ApplicationConfig config) {
        return config != null ? ClientFactory.subscriberClient(config) : ClientFactory.subscriberClient();
    }

    private static SenderClient.Start getSenderClient(final ApplicationConfig config) {
        return config != null ? ClientFactory.senderClient(config) : ClientFactory.senderClient();
    }

    private static void printSplashScreen() {
        LOGGER.info("{}", getSplashScreenResource(LOGO));
    }

    private static String getSplashScreenResource(final String bannerName) {
        final InputStream is = CommandLineRunner.class.getResourceAsStream(bannerName);
        return new BufferedReader(new InputStreamReader(is)).lines().collect(Collectors.joining("\n"));
    }
}
