package dev.fitko.fitconnect.client.cmd.batch;

import com.opencsv.bean.CsvToBeanBuilder;
import dev.fitko.fitconnect.api.exceptions.BatchImportException;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.Reader;
import java.util.List;

public class CsvImporter implements BatchImporter {

    public static final char CSV_DELIMITER = ',';

    @Override
    public List<ImportRecord> readRecords(final String path) {
        try {
            final Reader reader = new BufferedReader(new FileReader(path));
            return readToImportRecords(reader);
        } catch (final FileNotFoundException | IllegalStateException e) {
           throw new BatchImportException(e.getMessage(), e);
        }
    }

    private List<ImportRecord> readToImportRecords(final Reader reader) {
        return new CsvToBeanBuilder<ImportRecord>(reader)
                .withType(ImportRecord.class)
                .withSeparator(CSV_DELIMITER)
                .withIgnoreLeadingWhiteSpace(true)
                .withIgnoreEmptyLine(true)
                .build()
                .parse();
    }
}
