package dev.fitko.fitconnect.client.strategies;

import dev.fitko.fitconnect.api.domain.model.submission.SubmitSubmission;
import dev.fitko.fitconnect.client.model.SubmissionPayload;

import java.util.Optional;

/**
 * Provides different strategies for assembling, announcing and handing in a new submission.
 */
public interface SubmitStrategy {

    /**
     * Send a submission based on specific assumptions on how the submission is assembled.
     *
     * @param submissionPayload all necessary data for creating and submitting a submission
     *
     * @return {@link SubmitSubmission}, empty if a validation error or technical error occurred.
     */
    Optional<SubmitSubmission> send(SubmissionPayload submissionPayload);

    /**
     * Validate the payload to be sufficient for submission.
     *
     * @param submissionPayload payload of all data needed to create a submission
     *
     * @return true if all checks apply, false if data is missing
     */
    boolean hasValidPayload(SubmissionPayload submissionPayload);
}
