package dev.fitko.fitconnect.core.destination;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.domain.auth.OAuthToken;
import dev.fitko.fitconnect.api.domain.model.destination.Destination;
import dev.fitko.fitconnect.api.domain.model.jwk.JWK;
import dev.fitko.fitconnect.api.exceptions.RestApiException;
import dev.fitko.fitconnect.api.services.auth.OAuthService;
import dev.fitko.fitconnect.api.services.destination.DestinationService;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.util.UUID;

public class DestinationRestService implements DestinationService {

    private final OAuthService authService;
    private final RestTemplate restTemplate;
    private final ApplicationConfig config;

    public DestinationRestService(final OAuthService authService, final RestTemplate restTemplate, final ApplicationConfig config) {
        this.authService = authService;
        this.restTemplate = restTemplate;
        this.config = config;
    }

    @Override
    public Destination getDestination(final UUID destinationID) {
        final String url = config.destinationEndpoint();
        final OAuthToken token = authService.getCurrentToken();
        return performDestinationRequest(token.getAccessToken(), url, Destination.class, destinationID);
    }

    @Override
    public JWK getEncryptionKey(final UUID destinationID, final String keyId) {
        final String url = config.destinationKeyEndpoint();
        final OAuthToken token = authService.getCurrentToken();
        return performDestinationRequest(token.getAccessToken(), url, JWK.class, destinationID, keyId);
    }

    private <T> T performDestinationRequest(final String accessToken, final String url, final Class<T> responseType, final Object... params) {
        final HttpHeaders headers = getHeaders(accessToken);
        final HttpEntity<String> entity = new HttpEntity<>(headers);
        try {
            return restTemplate.exchange(url, HttpMethod.GET, entity, responseType, params).getBody();
        } catch (final RestClientException e) {
            throw new RestApiException("Could not retrieve destination", e);
        }
    }

    private HttpHeaders getHeaders(final String oAuthToken) {
        final var headers = new HttpHeaders();
        headers.setBearerAuth(oAuthToken);
        return headers;
    }
}
