package dev.fitko.fitconnect.core.events;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.domain.model.event.EventLog;
import dev.fitko.fitconnect.api.exceptions.RestApiException;
import dev.fitko.fitconnect.api.services.auth.OAuthService;
import dev.fitko.fitconnect.api.services.events.EventLogService;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.UUID;

public class EventLogApiService implements EventLogService {

    private final OAuthService authService;
    private final RestTemplate restTemplate;
    private final ApplicationConfig config;

    public EventLogApiService(final OAuthService authService, final RestTemplate restTemplate, final ApplicationConfig config) {
        this.authService = authService;
        this.restTemplate = restTemplate;
        this.config = config;
    }

    @Override
    public EventLog getEventLog(final UUID caseId) {
        final String url = config.getEventLogEndpoint();
        final HttpHeaders headers = getHttpHeaders();
        final HttpEntity<String> entity = new HttpEntity<>(headers);
        try {
            return restTemplate.exchange(url, HttpMethod.GET, entity, EventLog.class, caseId).getBody();
        } catch (final RestClientException e) {
            throw new RestApiException("EventLog query failed", e);
        }
    }

    @Override
    public void sendEvent(final UUID caseId, final String signedAndSerializedSET) {
        final String url = config.getEventLogEndpoint();
        final HttpHeaders headers = getHttpHeaders("application/jose");
        final HttpEntity<String> entity = new HttpEntity<>(signedAndSerializedSET, headers);
        try {
            restTemplate.exchange(url, HttpMethod.POST, entity, Void.class, caseId);
        } catch (final RestClientException e) {
            throw new RestApiException("Confirming submission failed", e);
        }
    }

    private HttpHeaders getHttpHeaders(final String contentType) {
        return getHttpHeaders(MediaType.parseMediaType(contentType));
    }

    private HttpHeaders getHttpHeaders() {
        return getHttpHeaders(MediaType.APPLICATION_JSON);
    }

    private HttpHeaders getHttpHeaders(final MediaType mediaType) {
        final var headers = new HttpHeaders();
        headers.setBearerAuth(authService.getCurrentToken().getAccessToken());
        headers.setContentType(mediaType);
        headers.setAcceptCharset(List.of(StandardCharsets.UTF_8));
        return headers;
    }

}
